c ==================================================================
c  1 compilacion KAI [notar fuente extension *.f]
c    f95 -w90 -o cbandapr.exe cbandapr.f
c
c  2 compilacion PGI por etapas [notar fuente extension *.hpf]:
c    pghpf -V -fast -Mautopar -Mmpi -c cbandapr.hpf
c    pghpf -V -fast -Mautopar -Mmpi -o cbandapr.exe cbandapr.o
c
c  3 compilacion PGI resumida  [notar fuente extension *.hpf]:
c    pghpf -V -fast -Mautopar -Mmpi -o cbandapr.exe cbandapr.hpf 
c
c  4 corrida con MPI
c    mpirun -machinefile machi.dat -nolocal -np 2 cbandapr.exe
c
c ------------------------------------------------------------------
        program  cbandapr
        implicit none
        integer, dimension (:,:), allocatable :: a
        integer, dimension   (:), allocatable :: x, b
        integer :: n, m1, m2, m
        integer :: i, j, k, ier
c ------------------------------------------------------------------
c construye un ejemplo simple: una matriz A de 5 por 5, con una
c sub-diagonal (m1 = 1) y con dos supra-diagonales (m2 = 2), pero
c es guardada en el formato comprimido  A_c de 5 filas y 3 columnas,
c una por cada diagonal.
c ------------------------------------------------------------------
        n  = 5           ! nro de filas
        m1 = 1           ! nro de   sub-diagonales
        m2 = 2           ! nro de supra-diagonales
        m  = m1 + 1 + m2 ! nro de col.: suma 1 por la diagonal ppal
c ------------------------------------------------------------------
c   aloca e inicializa matriz A y vectores x, b
c ------------------------------------------------------------------
        allocate ( a (n,m), x (n), b (n), stat = ier)
        if (ier .ne. 0)  stop' aloca '
        k = 0
        do  i = 1, n
        do  j = 1, m   ; a (i,j) = k ; k = k + 1 ; end do
        end do
        a (1,1) = 0    ; k = 0
        do  i = n-1, n
        do  j = i-k, m ; a (i,j) = 0 ; end do
        k = k + 2
        end do
        forall (i=1:n)  
          x (i) = n - i + 1
          b (i) = 0
        end forall
c ------------------------------------------------------------------
c   impresion eco
c ------------------------------------------------------------------
        write (*,*)
        write (*,*)'nro de filas matriz comprimida ;  n = ', n  
        write (*,*)'nro de colu. matriz comprimida ;  m = ', m  
        write (*,*)'nro de   sub-diagonales        ; m1 = ', m1 
        write (*,*)'nro de supra-diagonales        ; m2 = ', m2  
        call  eco2 (a, 'matriz A en formato comprimido')
        call  eco1 (x, 'vector X a multiplicar')
c ------------------------------------------------------------------
c   realiza multiplicacion matriz_comprimida por vector
c ------------------------------------------------------------------
        call  cbanda (a, m1, m2, x, b)
c ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
        contains
c -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
        subroutine  cbanda (a, m1, m2, x, b)
        implicit none
        integer, dimension (:,:), intent (inout) :: a
        integer                 , intent (in)    :: m1, m2
        integer, dimension (:)  , intent (in)    :: x
        integer, dimension (:)  , intent (out)   :: b
        integer, dimension (size(a,1),size(a,2)) :: y
        integer, dimension (size(a,2))           :: u
        character (64) :: cad
        integer        :: m, n, i
c ..................................................................
        cad = 'error (cbanda): m <> m1+m2+1 '
        n = size (a,1)
        m = siza2 ( size (a,2), m1+m2+1, cad)
c ..................................................................
        y = spread (x, dim = 2, ncopies = m)
        call  eco2 (y, 'matriz Y con el vector X expandido')
c ..................................................................
        u = prog_arit (-m1,m,1)
        call  eco1 (u, 'vector U con la progresion aritmetica')
c ..................................................................
        y = eoshift (y, dim = 1, shift = u)
        call  eco2 (y, 'matriz Y con el vector X corrido segun U')
c ..................................................................
        b = sum (a * y, dim = 2)
        call  eco1 (b, 'vector B con el producto matriz-vector')
c ..................................................................
        end subroutine
c -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
c control en la coherencia de las dimens. de 2 arreglos matriciales
c ------------------------------------------------------------------
        integer function siza2 (n1, n2, cad)
        integer      , intent (in) :: n1, n2
        character (*), intent (in) :: cad
        if ( n1 .ne. n2 ) then
           write (*,*)
           write (*,'(a)') cad
           stop
        end if
        siza2 = n1
        end function
c -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
c   sale "veces" elementos de la progresion aritmetica que empieza
c   en "ini" y de incremento "delta"
c ------------------------------------------------------------------
        function prog_arit (ini, veces, delta)
        integer      , intent (in) :: ini, veces, delta
        integer, dimension (veces) :: prog_arit
        integer :: k
        prog_arit = 0 
        if (veces > 0) prog_arit (1) = ini
        do  k = 2, veces
          prog_arit (k) = prog_arit (k-1) + delta
        end do
        end function
c -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
        subroutine eco2 (t, s)
        integer, dimension (:,:), intent (in) :: t
        character (*)           , intent (in) :: s
        integer :: i, m, n
        m = size (t,1)
        n = size (t,2)
        write (*,*)
        write (*,*) s
        do  i = 1, m
        write (*,*) t (i,:)
        end do
        read (*,*)
        end subroutine
c -----+-----+-----+-----+-----+-----+-----+-----+-----+-----+-----
        subroutine eco1 (t, s)
        integer, dimension (:), intent (in) :: t
        character (*)         , intent (in) :: s
        write (*,*)
        write (*,*) s
        write (*,*) t
        read  (*,*)
        end subroutine
c ..................................................................
        end program
c ==================================================================
