c -*- mode: fundamental -*-
c
c pgf90 -V -Mmp -o omp_d02.exe omp_d02.f
c
c -----------------------------------------------------------------
c
c 2 Directive: DO
c
c The DO directive specifies that the iterations of the 
c immediately following DO loop must be executed in parallel. 
c The DO directive must be enclosed in a parallel region; it 
c creates no threads by itself. The following do loop can
c not be a DO WHILE.
c
c !$OMP DO  [clause[[,]clause ...]
c
c         do_loop
c
c !$OMP END DO [NOWAIT]
c
c The clause can be one of these.
c
c It is illegal to branch out of a DO loop associated with 
c the DO directive.
c
c Example Filename: dodir.f
c
c Notice that the loop was divided among the 4 threads as we 
c requested with the SCHEDULE (STATIC,4) clause. Also note that 
c if had not enclosed the do loop in a DO/END DO directive block, 
c the loop would not have been split, but would have been 
c executed OMP_NUM_THREADS number of times.
c -----------------------------------------------------------------
        program dodir
        implicit none
        integer, parameter :: idp = kind (1.0 d0)
        integer, parameter :: iin = kind (1)
c .................................................................
        real    (idp), parameter ::   cero = 0.0 d0
        real    (idp), parameter ::    uno = 1.0 d0
        real    (idp), parameter ::    dos = 2.0 d0
        integer (iin), parameter :: n = 16
c .................................................................
        integer :: omp_get_num_threads, np
        integer :: omp_get_thread_num , tn
c .................................................................
        real    (idp), dimension (n) :: a, b
        real    (idp) :: s
        integer (iin) :: i, j
c -----------------------------------------------------------------
        call random_number (a)
        call random_number (b)
c -----------------------------------------------------------------
!$omp   parallel default (private) shared (a,b)
!$omp   do schedule (static, 4)
        do  i = 2, n
        b (i) = ( a (i) - a (i - 1) ) / dos
        np = omp_get_num_threads ()
        tn = omp_get_thread_num  ()
        write (*,*)' tarea', tn, ' de', np, ' tiene', i
        end do ! i
!$omp   end do
!$omp   end parallel
c -----------------------------------------------------------------
        s = maxval (b)
        j = maxloc (b, 1)
        write (*,*)
        write (*,*)' gradiente maximo = ', s
        write (*,*)' en posicion    j = ', j
        write (*,*)
        end program
c =================================================================
