c -*- mode: fundamental -*-
c
c pgf90 -V -Mmp -o omp_d03.exe omp_d03.f
c
c -----------------------------------------------------------------
c
c 3 Directive: PARALLEL DO
c
c The PARALLEL DO directive provides a shortcut form for 
c specifying a parallel region that contains a single DO 
c directive. The semantics are identical to specifying a 
c PARALLEL directive followed by a DO directive.
c
c  !$OMP PARALLEL DO  [clause[[,]clause ...]
c         do_loop
c  !$OMP END PARALLEL DO
c
c The clause can be any of those associated with the 
c PARALLEL and DO directives described above.
c
c Example
c
c !Filename: pardo.f
c !
c -----------------------------------------------------------------
        program pardo
        implicit none
        integer, parameter :: idp = kind (1.0 d0)
        integer, parameter :: iin = kind (1)
c .................................................................
        real    (idp), parameter ::   cero = 0.0 d0
        real    (idp), parameter ::    uno = 1.0 d0
        real    (idp), parameter ::    dos = 2.0 d0
        integer (iin), parameter :: n = 100
        integer (iin), parameter :: m = 200
c .................................................................
        integer :: omp_get_num_threads, np
        integer :: omp_get_thread_num , tn
c .................................................................
        real    (idp), dimension (n)   :: a
        real    (idp), dimension (m,n) :: b
        real    (idp), dimension (m,n) :: c
        real    (idp) :: s
        integer (iin) :: i, j
c -----------------------------------------------------------------
        call random_number (a)
c .................................................................
!$omp   parallel do schedule (runtime) 
!$omp&  private (i,j) shared (a, b, c, nt)
c .................................................................
        do j = 1, m
        tn = omp_get_num_threads ()
        do i = 2, n
        b (j,i) = ( (a (i) + a (i - 1) ) / dos ) / sqrt ( a (i) )
        c (j,i) = sqrt ( dos * a (i) ) / ( a (i) - a (i) / dos )
        b (j,i) = c (j,i) * ( b (j,i)**2 ) * sin ( a (i) )
        end do
        end do
!$omp   end parallel do
        s = maxval (b)
        write (*,*)
        write (*,*)' maximo de B      = ', s
        write (*,*)
        end program
c =================================================================
