c -*- mode: fundamental -*-
c
c pgf90 -V -Mmp -o omp_d05.exe omp_d05.f
c
c -----------------------------------------------------------------
c
c 5 Directive: SINGLE
c
c The SINGLE directive specifies that the enclosed code is to be 
c executed by only one thread in the team. Threads that are not 
c executing in the SINGLE directive wait at the END SINGLE 
c directive unless NOWAIT is specified. It is illegal to branch 
c out of a SINGLE block.
c
c
c  !$OMP SINGLE  [clause[[,]clause ...]
c         block
c  !$OMP END SINGLE [NOWAIT]
c
c The clause can be on of the following:
c
c   * PRIVATE (list)
c   * FIRSTPRIVATE (list)
c
c Example
c
c Filename: single.f
c
c This shows use of the SINGLE directive.
c
c -----------------------------------------------------------------
        program single 
        implicit none
        integer, parameter :: idp = kind (1.0 d0)
        integer, parameter :: iin = kind (1)
c .................................................................
        real    (idp), parameter :: cero = 0.0 d0
        real    (idp), parameter ::  uno = 1.0 d0
        integer (iin), parameter ::    n = 5 
c .................................................................
        real    (idp), dimension (n) :: a, b, c, d
        real    (idp) :: suma
        integer (iin) :: i
c -----------------------------------------------------------------
!$omp   parallel shared (a,b,c,d) private (i)
!$omp   sections
c .................................................................
!$omp   section
        open (10, file = 'a1.dat', status = 'old')
        read (10,*) ( a (i), i = 1, n)
c .................................................................
!$omp   section
        open (11, file = 'a2.dat', status = 'old')
        read (11,*) ( b (i), i = 1, n)
c .................................................................
!$omp   section
        open (12, file = 'a3.dat', status = 'old')
        read (12,*) ( c (i), i = 1, n)
!$omp   end sections
c .................................................................
!$omp   single
        suma = sum (a) + sum (b) + sum (c)
        write (*,*)
        write (*,*)' sum (A + B + C) = ', real (suma)
!$omp   end single 
!$omp   do schedule (static, 4)
        do  i = 1, n
        d (i) = a (i) + b (i) * c (i)
        end do ! i 
!$omp   end do
!$omp   end parallel
        write (*,*)
        write (*,*)' los valores de D son '
        do  i = 1, n
        write (*,*) real ( d (i) )
        end do ! i 
        write (*,*)
        end program
c =================================================================
