c -*- mode: fundamental -*-
c
c pgf90 -V -Mmp -o omp_d06.exe omp_d06.f
c
c -----------------------------------------------------------------
c
c 6 Directive: MASTER
c
c The code enclosed with MASTER and END MASTER directives is 
c executed only by the master thread of the team. There is no 
c implied barrier either on entry to or exit from the MASTER 
c section. Branching out of a MASTER block is illegal.
c
c
c  !$OMP MASTER
c         block
c  !$OMP END MASTER
c
c Example (Bad):
c
c The following example will cause problems without the use of 
c a critical region.
c
c Filename: critical1.f
c
c This shows an incorrect running without the CRITICAL directive.
c
c We can see the problem by compiling and running.
c
c setenv OMP_NUM_THREADS 16
c xlf90_r -qsmp=omp critical.f
c ./a.out
c Number of threads: 16
c The final value of L is 19
c ./a.out
c Number of threads: 16
c The final value of L is 15
c ./a.out
c Number of threads: 16
c The final value of L is 13
c ./a.out
c Number of threads: 16
c The final value of L is 12
c ./a.out
c Number of threads: 16
c The final value of L is 14
c ./a.out
c Number of threads: 16
c The final value of L is 13
c
c Not only do we not get the expected answer of 26, but 
c we get a different answer every time. What's the problem ?
c
c The error occurs when we call the subroutine ADD_ONE, 
c which adds one to the shared variable L. Each thread passes 
c the address of L at the time it gets to the ADD_ONE call. 
c But the order in which that happens is undefined. One
c thread makes the call while another is the process of doing 
c the addition but has not yet updated the value. The result 
c is indeterminate. (The "Hi Mom!" write statement is there 
c just to ensure that the threads get well out of synch.)
c
c We can fix the problem (although perhaps not efficiently) 
c by allowing only one thread at a time to update the value of L. 
c We do this by using a CRITICAL region.
c 
c -----------------------------------------------------------------
        program  master
        implicit none
        integer, parameter :: idp = kind (1.0 d0)
        integer, parameter :: iin = kind (1)
c .................................................................
        integer (iin) :: omp_get_num_threads, np
        integer (iin) :: i, j
c -----------------------------------------------------------------
        j = 10
!$omp   parallel shared (j) private (np, i)
!$omp   master
        np = omp_get_num_threads ()
        write (*,*)
        write (*,*)' nro de procesos ; np ', np
!$omp   end master
c -----------------------------------------------------------------
c   *** The following is going to cause problems without a
c   *** CRITICAL directive
c -----------------------------------------------------------------
        call add_one (j)
!$omp   end parallel
        write (*,*)
        write (*,*)' valor final de j = ', j
        contains
          subroutine add_one (i)
          implicit none
          integer (iin), intent (inout) :: i
          integer (iin) :: j
          j = i
!$omp     master
          open (1, file = 'salida.tmp', status = 'unknown')
          do i = 1, 10
          write (1,*)' Hi Mom ! '
          end do ! i
          close (1)
!$omp     end master
          j = j + 1
          i = j
          end subroutine add_one
        end program
c =================================================================
