c -*- mode: fundamental -*-
c
c pgf90 -V -mmp -o omp_d10.exe omp_d10.f
c
c -----------------------------------------------------------------
c
c 10 Directive: ATOMIC
c
c  !$omp atomic
c
c the atomic directive ensures that a specific memory location is 
c to be updated atomically, rather than exposing it to the 
c possibility of multiple, simultaneous writing threads.
c
c example
c
c filename: density.f
c
c -----------------------------------------------------------------
        program density
        implicit none
c .................................................................
        integer, parameter :: nbins=10
        integer, parameter :: nparticles=100000
        real :: xmin
        real :: xmax
        real :: maxmass
        real :: minmass
c -----------------------------------------------------------------
        real   , dimension (nparticles) :: x_location
        real   , dimension (nparticles) :: particle_mass
        integer, dimension (nparticles) :: bin
        real   , dimension (nbins)      :: grid_mass
        real   , dimension (nbins)      :: grid_density
        integer, dimension (nbins)      :: grid_n
c -----------------------------------------------------------------
        real    :: dx
        real    :: dxinV
        real    :: total_mass
        real    :: check_mass
        integer :: i
        integer :: check_n
        integer :: xmax_loc (1)
c -----------------------------------------------------------------
        grid_mass  = 0.0
        total_mass = 0.0
        grid_n     = 0
        check_mass = 0.0
        check_n    = 0
c -----------------------------------------------------------------
c initialize particle positions and masses
c -----------------------------------------------------------------
        call random_number (particle_mass)
        call random_number (x_location)
c -----------------------------------------------------------------
        maxmass = maxVal (particle_mass)
        minmass = minVal (particle_mass)
        xmax = maxVal (x_location)
        xmin = minVal (x_location)
        write (*,*)' minmass = ', minmass
        write (*,*)' maxmass = ', maxmass
        write (*,*)' xmin    = ', xmin
        write (*,*)' xmax    = ',xmax
c -----------------------------------------------------------------
c grid spacing (and inverse)
c -----------------------------------------------------------------
        dx    = (xmax - xmin) / float (nbins)
        dxinV = 1 / dx
c -----------------------------------------------------------------
!$omp   parallel default (shared) priVate (i) 
!$omp&  reduction (+: total_mass)
!$omp   do
        do i = 1, nparticles
        if ( i == xmax_loc (1) ) then
          bin (i) = nbins
        else
          bin (i) = 1 + (x_location (i) - xmin) * dxinv
        end if
        if ( bin (i) < 1 .or. bin (i) > nbins ) then
          write (*,*)' error: fuera de la grilla '
          write (*,*)'        bin = ', bin (i)
          write (*,*)'        x   = ', x_location (i)
        else
!$omp     atomic
          grid_mass ( bin (i) ) =
     #    grid_mass ( bin (i) ) + particle_mass (i)
!$omp     atomic
          grid_n ( bin (i) ) = grid_n ( bin (i) ) + 1
          total_mass = total_mass + particle_mass (i)
        end if
        end do
!$omp   end do
!$omp   end parallel
        do i=1, nbins
        grid_density (i) = grid_mass (i) * dxinv
        end do
c -----------------------------------------------------------------
        write (*,*)' total particles = ', nparticles
        write (*,*)' total mass      = ',total_mass
        do  i = 1, nbins
        write (*,*)' density (',i,' ) = ', grid_density (i)
        write (*,*)' mass    (',i,' ) = ', grid_mass    (i)
        end do
c -----------------------------------------------------------------
c check for consistency
c -----------------------------------------------------------------
        do i = 1, nbins
        check_mass = check_mass + grid_mass(i)
        check_n    = check_n + grid_n(i)
        end do
c -----------------------------------------------------------------
        write (*,*)' particles  on grid = ', check_n
        write (*,*)' total mass on grid = ', check_mass
        write (*,*)
        end program
c -----------------------------------------------------------------
