c -*- mode: fundamental -*-
c
c pgf90 -V -Mmp -o ordered.exe ordered.f
c
c -----------------------------------------------------------------
c
c 11 Directive: ORDERED
c
c The code enclosed with ORDERED and END ORDERED directives is 
c executed in the order in which iterations would be executed in 
c a sequential execution of the loop. The ORDERED directive can 
c only appear in the context of a DO or PARALLEL DO directive. 
c It is illegal to branch into or out of an ORDERED block.
c
c  !$OMP ORDERED
c         block
c  !$OMP END ORDERED
c
c Example
c
c -----------------------------------------------------------------
        program ordered
        implicit none
        integer, parameter :: idp = kind (1.0 d0)
        integer, parameter :: iin = kind (1)
c .................................................................
        real    (idp), parameter ::   cero = 0.0 d0
        real    (idp), parameter ::    uno = 1.0 d0
        integer (iin), parameter :: n = 1 000
        real    (idp), parameter :: m = 1 000
c .................................................................
        integer       :: omp_get_thread_num
        integer       :: np
c .................................................................
        real    (idp), dimension (n,m) :: x, y
        real    (idp), dimension (n)   :: z
        integer (iin) :: i, j
c -----------------------------------------------------------------
        call random_number (x)
        call random_number (y)
        z = cero
c -----------------------------------------------------------------
        write (*,*)
        write (*,*)' Primeros 10 valores de Z: '
c -----------------------------------------------------------------
!$omp   parallel default (shared) private (i,j)
!$omp   do schedule (dynamic, 4) ordered
          do  i = 1, n
            do  j = 1, m
            z (i) = z (i) + x (i,j) * y (j,i)
            end do ! j
!$omp       ordered
            if  (i < 11 ) write (*,*)' i z = ', i, z (i)
!$omp       end ordered
          end do ! i
!$omp   end do
!$omp   end parallel
c -----------------------------------------------------------------
        write (*,*)
        end program
c =================================================================
