// -----------------------------------------------------------------
// pgCC -V -Minfo -mp -fast -c omp_hola.cc
// pgCC -V -Minfo -mp -fast -o omp_hola_cc.exe omp_hola.o
// omp_hola_cc.exe
//
// -----------------------------------------------------------------
// OpenMP Ejemplo - hola mundo - C/C++ Version
// FILE: omp_hola.cc
// DESCRIPTION:
//  In this simple example, the master thread forks a parallel 
//  region. All threads in the team obtain their unique thread 
//  number and print it. The master thread only prints the total 
//  number of threads.  Two OpenMP library routines are used to 
//  obtain the number of threads and each thread's number.
// SOURCE: adapted from Blaise Barney  5/99
// -----------------------------------------------------------------
// 1 Directive: PARALLEL
//
// A Parallel Region is a block of code that is to be executed 
// in parallel by a number of threads. Each thread executes the 
// enclosed code separately.
//
// Note that all code within a parallel region is executed by each 
// thread unless other OpenMP directives specify otherwise. For 
// instance, a DO loop that lies within a parallel region will be 
// executed completely by each thread unless a parallel DO 
// directive is inserted before the loop. A DO or PARALLEL DO 
// directive is necessary if you want the loop to be executed 
// once, but with various iterations performed in parallel.
//
// It is illegal to branch out of a Parallel Region.
//
// # pragma omp parallel [clauses]
// {
//    code block
//
// } // end prgama
//
// The clause can be any legal one.
// -----------------------------------------------------------------
#include <iostream>
#include <omp.h>
main ( ) {
  int  ih, nh ;
  cout << endl ;
  // bifurca (fork) un grupo de "hilos" (threads) o tareas
  // en paralelo donde cada uno tiene su propia copia de variables
#pragma omp parallel private (ih, nh)
  {
    // Obtiene el identificador de cada hilo (thread)
    ih = omp_get_thread_num ();
    cout << "hola desde el filamento (thread) = " << ih << endl ;
    // Solo el filamento master hace lo siguiente 
    if (ih == 0) {
      nh = omp_get_num_threads ();
      cout << "numero de hilos = " << nh << endl ;
    } // end if
  } // end pragma: todos los hilos se unen al master y finalizan
} // end main
// -----------------------------------------------------------------
