c =================================================================
c pgf90 -V -Minfo -mp -fast -c omp_hola.f
c pgf90 -V -Minfo -mp -fast -o omp_hola_ff.exe omp_hola.o
c omp_hola_ff.exe
c
c -----------------------------------------------------------------
c OpenMP Ejemplo - hola mundo - Fortran Version
c FILE: omp_hola.f
c
c DESCRIPTION:
c  In this simple example, the master thread forks a parallel 
c  region. All threads in the team obtain their unique thread 
c  number and print it. The master thread only prints the total 
c  number of threads.  Two OpenMP library routines are used to 
c  obtain the number of threads and each thread's number.
c
c SOURCE: adapted from Blaise Barney  5/99
c
c -----------------------------------------------------------------
c 1 Directive: PARALLEL
c
c A Parallel Region is a block of code that is to be executed 
c in parallel by a number of threads. Each thread executes the 
c enclosed code separately.
c
c Note that all code within a parallel region is executed by each 
c thread unless other OpenMP directives specify otherwise. For 
c instance, a DO loop that lies within a parallel region will be 
c executed completely by each thread unless a parallel DO 
c directive is inserted before the loop. A DO or PARALLEL DO 
c directive is necessary if you want the loop to be executed 
c once, but with various iterations performed in parallel.
c
c It is illegal to branch out of a Parallel Region.
c
c $OMP PARALLEL  [clause]
c
c         code block
c
c !$OMP END PARALLEL
c
c The clause can be any legal one.
c -----------------------------------------------------------------
      program hola 
      implicit none
      integer  ih, nh
      integer  omp_get_thread_num 
      integer  omp_get_num_threads
      write (*,*)
c     bifurca (fork) un grupo de "hilos" (threads) o tareas
c     en paralelo donde cada uno tiene su propia copia de variables
!$omp parallel private (ih, nh)
c     Obtiene el identificador de cada hilo (thread)
      ih = omp_get_thread_num ()
      write (*,*)'hola desde el filamento (thread) = ', ih
c     Solo el filamento master hace lo siguiente 
      if (ih .EQ. 0) then
        nh = omp_get_num_threads ()
        write (*,*)'numero de hilos = ', nh
      end if
c     todos los hilos se unen al hilo master y finalizan
!$omp end parallel
      end program
c =================================================================
