// -----------------------------------------------------------------
// pgCC -V -Minfo -mp -fast -c omp_matvec.cc
// pgCC -V -Minfo -mp -fast -o omp_matvec_cc.exe omp_matvec.o
// omp_matvec_cc.exe
//
// -----------------------------------------------------------------
// OpenMP Example - Matrix-vector multiplication - C/C++ Version
//
// FILE: omp_matvec.cc
//
// DESCRIPTION:
//   This example multiplies all row i elements of matrix A with 
//   vector element b (i) and stores the summed products in vector 
//   c (i). A total is maintained for the entire matrix.  
//   Performed by using the OpenMP loop work-sharing construct.  
//   The update of the shared global total is serialized by 
//   using the OpenMP critical directive.
// SOURCE: modified from Blaise Barney  5/99
// -----------------------------------------------------------------
#include <omp.h>
#include <iostream>
#define n 10
main () {
  float a [n] [n], b [n], c [n], total;
  int   i, j, ih;
  // inicializa
  total = 0.0;
  for (i = 0 ; i < n ; i++) {
    for (j = 0 ; j < n ; j++) a [i] [j] = (j + 1) * 1.0 ;
    b [i] = 1.0 * (i + 1) ;
    c [i] = 0.0 ;
  } // end for
  cout << endl ;
  cout << "valores iniciales de A y B" << endl ;
  for (i = 0 ; i < n ; i++) {
    for (j = 0 ; j < n ; j++) {
      cout << " i = " << i << " j = " << j 
           << " a_ij = " << a [i] [j] << endl ;
    } // end 
    cout << " i = " << i << " b [i] = " << b [i] << endl ;
  } // end for
  cout << endl ;
  cout << " resultados por columna/hilo" << endl ;
  // define un grupo de hilos (threads) con sus variables locales
#pragma omp parallel shared (a,b,c,total) private (ih,i)
  {
    ih = omp_get_thread_num ();
    // lazo de trabajo distribuido en las filas de A
#pragma omp for private (j)
    for (i = 0 ; i < n ; i++) {
      for (j = 0 ; j < n ; j++) c [i] = c [i] + a [i] [j] * b [i];
      // actualiza e imprime el valor total en forma serial
    #pragma omp critical
      {
        total = total + c [i];
        cout << endl ;
        cout << "el hilo         ; ih    = " << ih    << endl ;
        cout << "hizo la columna ;  i    = " << i     << endl ;
        cout << "con             ; c [i] = " << c [i] << endl ;
        cout << "total           ; total = " << total << endl ;
      } // end pragma critical
    } // end for
  } // end pragma parallel
  cout << endl ;
  cout << " sum (c), con c=matriz*vector = " << total << endl ;
} // end main
// -----------------------------------------------------------------
