c =================================================================
c pgf90 -V -Minfo -mp -fast -c omp_matvec.f
c pgf90 -V -Minfo -mp -fast -o omp_matvec_ff.exe omp_matvec.o
c omp_matvec_ff.exe
c
c -----------------------------------------------------------------
c OpenMP Example - Matrix-vector multiplication - Fortran Version
c
c FILE: omp_matvec.f
c
c DESCRIPTION:
c   This example multiplies all column I elements of matrix A with vector
c   element B(I) and stores the summed products in vector C(I).  A total is
c   maintained for the entire matrix.  Performed by using the OpenMP loop
c   work-sharing construct.  The update of the shared global total is
c   serialized by using the OpenMP critical directive.
c
c SOURCE: adapted from Blaise Barney  5/99
c -----------------------------------------------------------------
      program matvec
      implicit none
      integer, parameter :: n = 10
      real, dimension (n,n) :: a
      real, dimension (n)   :: b, c
      real                  :: total
      integer :: i, j, ih
      integer :: omp_get_thread_num
c     inicializa
      total = 0.0
c     call random_number (a)
c     call random_number (b)
      c = 0.0
      do i = 1, n
        do j = 1, n
          a (j,i) = j * 1.0
        end do
        b (i) = i
      end do
      write (*,*)
      write (*,*)'valores iniciales de A y B '
      do  i = 1, n
        do  j = 1, n
        write (*,*)' i j a_ij ', i, j, a (i,j)
        end do ! j
      end do ! i
      do  i = 1, n
        write (*,*)' i   b_i  ', i,    a (i,j)
      end do
      write (*,*)
      write (*,*)' resultados por columna/hilo'
c     crea una zona de hilos (threads) con sus variables locales
c$omp parallel shared (a,b,c,total) private (i,ih)
      ih = omp_get_thread_num ()
c     lazo de trabajo distribuido en las columnas de A
c$omp do private (j)
      do i = 1, n
        do j = 1, n
          c (i) = c (i) + a (j,i) * b (i)
        end do
c       actualiza e imprime el valor total en forma serial
c$omp   critical
        total = total + c (i)
        write (*,*)
        write (*,*)' el hilo         ; ih    = ', ih
        write (*,*)' hizo la columna ; i     = ', i
        write (*,*)' con             ; c (i) = ', c (i)
        write (*,*)' total           ; total = ', total
c$omp end critical
      end do
c$omp end do
c$omp end parallel
      write (*,*)
      write (*,*)' sum (c), con c=matriz*vector = ', total
      end program
c =================================================================
