// -----------------------------------------------------------------
// pgCC -V -Minfo -mp -fast -c omp_orphan.cc
// pgCC -V -Minfo -mp -fast -o omp_orphan_cc.exe omp_orphan.o
// omp_orphan_cc.exe
//
// -----------------------------------------------------------------
// OpenMP Example - Parallel region with an orphaned directive -
//                  C/C++ Version
//
// FILE: omp_orphan.cc
//
// DESCRIPTION:
//   This example demonstrates a dot product  being performed by 
//    an orphaned loop reduction construct. Scoping of the reduction
//    variable is critical.
//
// SOURCE: modified from Blaise Barney  5/99
// -----------------------------------------------------------------
#include <omp.h>
#include <iostream>
#define n 10
float a [n], b [n] ;

// -----------------------------------------------------------------
float dotprod () {
  float sum ;
  int i, ih;
  ih = omp_get_thread_num ();
  sum = 0.0 ;
#pragma omp for reduction (+: sum)
  for (i = 0 ; i < n ; i++) {
    sum = sum + a [i] * b [i] ;
    cout << "  ih = " << ih << " i = " << i << endl ;
  } // end for
  cout << endl ;
  return sum ;
} // end float

// -----------------------------------------------------------------
main () {
  int    i ;
  float  sum ;
  for (i = 0 ; i < n ; i++)  {
     a [i] = 1.0 * i ;
     b [i] = 2.0 * i ;
  } // end for
#pragma omp parallel
  sum = 0 ;
  sum = dotprod () ;
  cout << endl ;
  cout << "sum = " << sum << endl ;
  cout << endl ;
} // end main
// -----------------------------------------------------------------
