c =================================================================
c pgf90 -V -Minfo -mp -fast -c omp_orphan.f
c pgf90 -V -Minfo -mp -fast -o omp_orphan_ff.exe omp_orphan.o
c omp_orphan_ff.exe
c
c -----------------------------------------------------------------
c
c OpenMP Example - Parallel region 
c with an orphaned directive - Fortran Version
c
c FILE: omp_orphan.f
c
c DESCRIPTION:
c   This example demonstrates a dot product being performed by an 
c   orphaned loop reduction construct.  Scoping of the reduction 
c   variable is critical.
c
c SOURCE: adapted from Blaise Barney  5/99
c -----------------------------------------------------------------
      program orphan
      implicit none
      integer, parameter :: n = 10
      real, dimension (n) :: a, b
      real :: sum
      integer :: i
      do  i = 1, n
        a (i) = 1.0 * i
        b (i) = a (i)
      end do
c$omp parallel
      call  dotprod (a, b, sum)
c$omp end parallel
      write (*,*)
      write (*,*)' suma = ', sum
      contains
        subroutine  dotprod (a, b, sum)
        real, dimension (:), intent (in)  :: a, b
        real               , intent (out) :: sum
        integer  omp_get_thread_num
        integer  n, i, ih
        n  = size (a)
        ih  = omp_get_thread_num ()
        sum = 0.0
c$omp   do reduction (+:sum)
        do  i = 1, n
          sum = sum + a (i) * b(i)
          write (*,*)'  ih  i = ', ih, i
        end do
        end subroutine
      end program
c =================================================================
